#![feature(if_let_guard, let_chains)]

pub mod cpu;
pub mod rt;

pub use crate::cpu::*;
pub use crate::rt::*;

use std::collections::BTreeMap;

use hercules_ir::*;

pub const LARGEST_ALIGNMENT: usize = 8;

/*
 * The alignment of a type does not depend on dynamic constants.
 */
pub fn get_type_alignment(types: &Vec<Type>, ty: TypeID) -> usize {
    match types[ty.idx()] {
        Type::Control => panic!(),
        Type::Boolean | Type::Integer8 | Type::UnsignedInteger8 => 1,
        Type::Integer16 | Type::UnsignedInteger16 => 2,
        Type::Integer32 | Type::UnsignedInteger32 | Type::Float32 => 4,
        Type::Integer64 | Type::UnsignedInteger64 | Type::Float64 => 8,
        Type::Product(ref members) | Type::Summation(ref members) => members
            .into_iter()
            .map(|id| get_type_alignment(types, *id))
            .max()
            .unwrap_or(1),
        Type::Array(elem, _) => get_type_alignment(types, elem),
    }
}

/*
 * Nodes producing collection values are "colored" with what device their
 * underlying memory lives on.
 */
pub type FunctionNodeColors = BTreeMap<NodeID, Device>;
pub type NodeColors = Vec<FunctionNodeColors>;

/*
 * The allocation information of each function is a size of the backing memory
 * needed and offsets into that backing memory per constant object and call node
 * in the function.
 */
pub type FunctionBackingAllocation =
    BTreeMap<Device, (DynamicConstantID, BTreeMap<NodeID, DynamicConstantID>)>;
pub type BackingAllocations = BTreeMap<FunctionID, FunctionBackingAllocation>;
pub const BACKED_DEVICES: [Device; 2] = [Device::LLVM, Device::CUDA];
